"""Tests for ckanext.{{ project_shortname }}.helpers module.

You can either call helpers directly or use `tk.h` and `with_plugins` fixture
to register and call helpers. The former is simpler, the latter verifies that
helpers are registered.
"""

from __future__ import annotations

from ckanext.{{project_shortname}} import helpers


def test_hello():
    """Helper returns expected greeting."""
    assert helpers.{{
        project_shortname
    }}_hello() == "Hello, {{project_shortname}}!"



from __future__ import annotations

import textwrap
from typing import TYPE_CHECKING

import click

if TYPE_CHECKING:
    from ckan.config.declaration import Declaration


@click.group()
def readme():
    """Generate fragments for README.md."""


@readme.command()
@click.argument("plugins", nargs=-1)
def config(
    plugins: tuple[str, ...],
):
    """Print declared config options for the given plugins."""

    from ckan.config.declaration import Declaration
    from ckan.config.declaration.serialize import handler

    if not plugins:
        click.secho("At leas one plugin must be specified", fg="red")
        raise click.Abort
    handler.register("ckanext-readme")(_ckanext_readme)

    decl = Declaration()
    for name in plugins:
        decl.load_plugin(name)

    if decl:
        click.echo(handler.handle(decl, "ckanext-readme"))


def _ckanext_readme(declaration: Declaration):
    from ckan.config.declaration import Flag

    result = ""

    for item in declaration.iter_options():
        option = declaration[item]

        if option.has_flag(Flag.non_iterable()):
            continue

        if option.description:
            result += (
                textwrap.fill(
                    option.description,
                    width=77,
                    initial_indent="# ",
                    subsequent_indent="# ",
                )
                + "\n"
            )

        if not option.has_default():
            value = option.placeholder or ""
        elif isinstance(option.default, bool):
            value = option.str_value().lower()
        else:
            value = option.str_value()

        if not option.has_flag(Flag.required):
            result += f"# (optional, default: {value})\n"

        result += f"{item} = {option.example or value}\n\n"

    return result



"""Definition of the main plugin.

If you have multiple plugins, it's recommended to create `plugins` module and
put every plugin into a separate submodule. Or even create a separate extension
for every plugin, if they are not really related.

It's possible to put multiple plugins into single module, but it may affect a
bit order of plugin in CKAN v2.10 because of implementation details of plugins
core. If order of plugins does not change anything, it's safe to define plugins
in the same file.

Blankets decorating the plugin are used to provide default implementation of
corresponding interfaces. When blanket is applied, it reads content from the
certain module and register it using CKAN Interface.

Source module for blanket is defined by CKAN recommendations. `cli` blanket
reads content from `cli.py`, `actions` blanket reads content from
`logic/action.py`, etc.

If you have a different project structure, you have two alternatives. Blankets
accept either a collection of items for corresponding interface, or a function
that can produce such collection.

Example:
    ```python
    def get_actions():
        return {"{{ project_shortname }}_something": something}

    helpers = {"{{ project_shortname }}_hello": hello}

    @tk.blanket.actions(get_actions)
    @tk.blanket.helpers(helpers)
    class {{ plugin_class_name }}(...):
        ...
    ```
"""

from __future__ import annotations

import ckan.plugins as p
import ckan.plugins.toolkit as tk
from ckan.common import CKANConfig

from . import implementations


@tk.blanket.actions
@tk.blanket.auth_functions
@tk.blanket.blueprints
@tk.blanket.cli
@tk.blanket.config_declarations
@tk.blanket.helpers
@tk.blanket.validators
class {{ plugin_class_name }}(
    # implementations are extracted to separate modules to keep main plugin
    # definition as lean as possible
    implementations.PackageController,
    # don't forget to extend SingletonPlugin. Due to internal
    # implementation details, it must be extended directly by the plugin
    p.SingletonPlugin,
):
    """Main entrypoint of the {{ project_shortname }} plugin.

    This plugin does nothing yet, but it will definitely grow into beautiful
    and useful plugin one day.
    """

    # it's still possible to implement interfaces directly inside the main
    # plugin. But do it only if implementation is really straightforward and
    # compact
    p.implements(p.IConfigurer)

    # IConfigurer
    def update_config(self, config_: CKANConfig):
        """Modify CKAN configuration."""
        # register templates of the plugin
        tk.add_template_directory(config_, "templates")

        # every file from the public directory can be accessed directly from
        # the browser. Use this for public images, site logos, downloadable
        # documents.
        tk.add_public_directory(config_, "public")

        # register assets folder. You must add `webassets.yml` into this folder
        tk.add_resource("assets", "{{ project_shortname }}")



"""Tests for ckanext.{{ project_shortname }}.logic.action."""

from __future__ import annotations

from typing import Any

import pytest

import ckan.model as model
import ckan.plugins.toolkit as tk
from ckan.tests.helpers import call_auth


@pytest.mark.ckan_config("ckan.plugins", "{{project_shortname}}")
@pytest.mark.usefixtures("with_plugins")
class TestGetSum:
    def test_anon_access(self):
        """Anonymous user can get sum."""
        assert call_auth(
            "{{project_shortname}}_get_sum",
            context={"model": model, "user": ""},
        )

    def test_authenticated_access(self, user: dict[str, Any]):
        """Anonymous user can get sum."""
        assert call_auth(
            "{{project_shortname}}_get_sum",
            context={"model": model, "user": user["name"]},
        )


@pytest.mark.ckan_config("ckan.plugins", "{{project_shortname}}")
@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestCreateSomething:
    def test_anon_access(self):
        """Anonymous user cannot create something."""
        with pytest.raises(tk.NotAuthorized):
            call_auth(
                "{{project_shortname}}_something_create",
                context={"model": model, "user": ""},
            )

    def test_authenticated_access(self, user: dict[str, Any]):
        """Authenticated user can create something."""
        assert call_auth(
            "{{project_shortname}}_something_create",
            context={"model": model, "user": user["name"]},
        )



from __future__ import annotations

import os

import click

from . import _shared


@click.group()
def gh_action():
    """Make GitHub actions."""


@gh_action.command()
@_shared.option_plugin
@_shared.option_write
def test(plugin: str, write: bool):
    """Test workflow."""
    _shared.ensure_root()
    _shared.produce(
        "action_test.yaml",
        _action_file("test"),
        {"PLUGIN": _shared.safe_plugin_name(plugin)},
        write,
    )


@gh_action.command()
@_shared.option_plugin
@_shared.option_write
def pypi_publish(plugin: str, write: bool):
    """Publish package to PyPI when vX.Y.Z tag added."""
    _shared.ensure_root()
    _shared.produce(
        "action_pypi-publish.yaml",
        _action_file("pypi-publish"),
        {"PLUGIN": _shared.safe_plugin_name(plugin)},
        write,
    )


@gh_action.command()
@_shared.option_plugin
@_shared.option_write
def release_please(plugin: str, write: bool):
    """Create a PR that compiles changelog records and publishes GitHub release."""
    _shared.ensure_root()
    _shared.produce(
        "action_release-please.yaml",
        _action_file("release-please"),
        {"PLUGIN": _shared.safe_plugin_name(plugin)},
        write,
    )


def _action_file(name: str) -> str:
    path = ".github/workflows"
    os.makedirs(path, exist_ok=True)
    return os.path.join(path, f"{name}.yml")



"""Tests for ckanext.{{ project_shortname }}.config.

You don't have to write tests for this module, unless it does something
complex.
"""

import pytest

from ckanext.{{ project_shortname }} import config


class TestOption:
    """Test option() accessor."""

    def test_default(self):
        """Default option is available."""
        assert config.option() == 10

    @pytest.mark.ckan_config(config.OPTION, 42)
    def test_modified(self):
        """Modified option is available."""
        assert config.option() == 42


class TestMultivalued:
    """Test multivalued() accessor."""

    def test_default(self):
        """Default option is available."""
        assert config.multivalued() == []

    @pytest.mark.ckan_config(config.MULTI, [1, 2, 3])
    def test_modified(self):
        """Modified option is available."""
        assert config.multivalued() == [1, 2, 3]



from setuptools import setup

setup(
    message_extractors={
        "ckanext": [
            ("**.py", "python", None),
            ("**.js", "javascript", None),
            ("**/templates/**.html", "ckan", None),
        ],
    },
)



"""Tests for Something model."""

from __future__ import annotations

from typing import Any

import pytest
from faker import Faker

from ckan import model

from ckanext.{{ project_shortname }}.model import Something


# Always include `with_plugins` before `clean_db`
@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestSomething:
    def test_by_hello(self, faker: Faker, something_factory: Any):
        """Filter `.by_hello` works."""
        first = something_factory()
        second = something_factory()

        smth = model.Session.scalar(Something.by_hello(first["hello"]))
        assert smth.id == first["id"]

        smth = model.Session.scalar(Something.by_hello(second["hello"]))
        assert smth.id == second["id"]

    def test_dictization(self, something: dict[str, Any]):
        """Plugin data excluded from dictized form by default."""
        smth = model.Session.get(Something, something["id"])
        assert smth.dictize({}) == {
            "id": something["id"],
            "hello": something["hello"],
            "world": something["world"],
        }

        assert smth.dictize({"include_plugin_data": True}) == {
            "id": something["id"],
            "hello": something["hello"],
            "world": something["world"],
            "plugin_data": smth.plugin_data,
        }



from __future__ import annotations

from typing import Any, Callable, TypeVar

import ckan.plugins.toolkit as tk

T = TypeVar("T")

__all__ = [
    "constantly",
    "config_getter",
]


def constantly(v: T) -> Callable[..., T]:
    return lambda *args, **kwargs: v


def config_getter(
    name: str,
    default: T | str | None,
    convert: Callable[[T | str | None], T] | None = None,
) -> Callable[..., T | str | None]:
    def getter(*_args: Any, **_kwargs: Any):
        v = tk.config.get(name, default)
        if convert:
            v = convert(v)
        return v

    return getter
